@@ -20,15 +20,7 @@ esac
 
 cd /opt
 unzip "/var/task/.perl-layer/dist/perl-$TAG-runtime-al2-$ARCH.zip"
-
 /opt/bin/cpanm --notest --no-man-pages Paws@0.44
 
-# install perlstrip
-# https://metacpan.org/pod/distribution/Perl-Strip/bin/perlstrip
-yum install -y parallel perl-App-cpanminus
-cpanm --notest Perl::Strip
-
-set +e # skip errors of stripping
-
-find /opt/lib/perl5/site_perl -type f -a -name '*.pm' -print0 | parallel -0 /var/task/author/perlstrip.sh
-find /opt/lib/perl5/site_perl -type f -a -name '*.pod' -print0 | xargs -0 rm
+# remove pods
+find /opt/lib/perl5/site_perl -type f -a -name '*.pod' -delete
